#!/usr/bin/python -ttE

import datetime
import glob
import os
import subprocess
import sys


# Output format (use .png or .jpg)
FILETYPE = '.png'

# ImageMagick programs
IM_CONVERT = 'convert'
IM_IDENTIFY = 'identify'


#                         W     H      TS  Prefix
SIZE_CONVERSIONS = [
#                    (  160,  120,   True, 'convert_ts_160x120_' ), # 160x120 thumbnail with timestamp (disabled)
                     (  160,  120,  False, 'convert_160x120_' ),    # 160x120 thumbnail without timestamp
                     (  640,  480,   True, 'convert_ts_640x480_' ), # 640x480 with timestamp
                     (  640,  480,  False, 'convert_640x480_' ),    # 640x480 without timestamp
                     (    0,    0,   True, 'convert_ts_sof_' ),     # Original size with timestamp
                   ]


#                               X    Y    W    H    #  Color
SCOREBOARD_HEADER =     [
                          [ (  32,  12,  40,   8,  25, 0xAFAFAF ) ],
                          [ (  88,  12,  16,   8,   5, 0xFFFFFF ),
                            ( 152,  12,   8,   8,   5, 0xFFFFFF ) ]
                        ]

SCOREBOARD_HEADER_CTF = [
                          [ (  32,  22,  32,   8,  20, 0xAFAFAF ) ],
                          [ ( 152,  22,   8,   8,   5, 0xFFFFFF ) ]
                        ]

SCOREBOARD_HEADER_CTL = [
                          [ (  32,  22,  40,   8,  25, 0xAFAFAF ) ],
                          [ (  88,  22,  16,   8,   5, 0xFFFFFF ) ]
                        ]

SCOREBOARD_CLIENT =     [
                          [ (  96,   8,  16,   8,   5, 0xFFFFFF ) ],
                          [ (  32,  16,  32,  16,  40, 0xAFAFAF ) ]
                        ]

# Set to 'True' or 'False' to enable / disable writing debug_* files
#SCOREBOARD_DEBUG = True
SCOREBOARD_DEBUG = False

# Font for timestamps
TS_FONT = 'DejaVuSansMono.ttf'
TS_SIZE = 10
TS_OFFSET_LEFT = 5
TS_OFFSET_BOTTOM = 5

BORDER_TOP = 5
BORDER_BOTTOM = 5
BORDER_LEFT = 5
BORDER_RIGHT = 5


def check_scoreboard_match(pathname, topleft_x, topleft_y, checks):
	for check in checks:
		match = False
		for (offset_x, offset_y, width, height, matches, color) in check:
			child = subprocess.Popen(
			                          [
			                            IM_CONVERT,
			                            pathname,
			                            '-crop',
			                            ''.join([str(width), 'x', str(height), '+', str(topleft_x + offset_x), '+', str(topleft_y + offset_y)]),
			                            'txt:-'
			                          ],
			                          stdout=subprocess.PIPE
			                        )
			output = child.communicate()[0]
			if child.returncode != 0:
				return False
			output = output.decode("utf-8")
			if output.count(''.join(['#', format(color, 'X')])) >= matches:
				match = True
				break
		if not match:
			return False
	return True


def check_scoreboard(pathname, scoreboard_topleft_x, scoreboard_topleft_y, scoreboard_info):
	# Check if first row contains a header
	match = check_scoreboard_match(pathname, scoreboard_topleft_x, scoreboard_topleft_y, SCOREBOARD_HEADER)
	if match:
		if scoreboard_topleft_x < scoreboard_info['min_x']:
			scoreboard_info['min_x'] = scoreboard_topleft_x
		if scoreboard_topleft_y < scoreboard_info['min_y']:
			scoreboard_info['min_y'] = scoreboard_topleft_y
		if scoreboard_topleft_x > scoreboard_info['max_x']:
			scoreboard_info['max_x'] = scoreboard_topleft_x
		if scoreboard_topleft_y > scoreboard_info['max_y']:
			scoreboard_info['max_y'] = scoreboard_topleft_y
		scoreboard_info['header_count'] = scoreboard_info['header_count'] + 1

		# Check if the header is a CTF header
		match = check_scoreboard_match(pathname, scoreboard_topleft_x, scoreboard_topleft_y, SCOREBOARD_HEADER_CTF)
		if match:
			scoreboard_info['is_ctf'] = True
		else:
			# Check if the header is a Control header
			match = check_scoreboard_match(pathname, scoreboard_topleft_x, scoreboard_topleft_y, SCOREBOARD_HEADER_CTL)
			if match:
				scoreboard_info['is_ctl'] = True

	# Check for players (start at row 3)
	y = scoreboard_topleft_y + ( 2 * 32 )
	row = 3
	while ( row <= 12 ):
		match = check_scoreboard_match(pathname, scoreboard_topleft_x, y, SCOREBOARD_CLIENT)
		if match:
			if scoreboard_topleft_x < scoreboard_info['min_x']:
				scoreboard_info['min_x'] = scoreboard_topleft_x
			if y < scoreboard_info['min_y']:
				scoreboard_info['min_y'] = y
			if scoreboard_topleft_x > scoreboard_info['max_x']:
				scoreboard_info['max_x'] = scoreboard_topleft_x
			if y > scoreboard_info['max_y']:
				scoreboard_info['max_y'] = y
			scoreboard_info['client_count'] = scoreboard_info['client_count'] + 1
		y = y + 32
		row = row + 1


def convert(pathname):
	pathname = os.path.normpath(pathname)
	(dirname, filename) = os.path.split(pathname)
	timestamp_name = filename[4:-4]  # Remove 'sof_' prefix and '.png' extension

	# Get original image information
	child = subprocess.Popen(
	                          [
	                            IM_IDENTIFY,
	                            '-format',
	                            '%w;%h;%[ssts]',
	                            pathname
	                          ],
	                          stdout=subprocess.PIPE
	                        )
	output = child.communicate()[0]
	if child.returncode != 0:
		sys.stderr.write('Unable to get image info\n')
		return
	output = output.decode("utf-8")
	(orig_width, orig_height, orig_timestamp) = output.split(';', 3)
	orig_width = int(orig_width)
	orig_height = int(orig_height)
	display_timestamp = ''.join([orig_timestamp[0:10], ' ', orig_timestamp[11:16]])

	# Convert image to different dimensions
	for (width, height, timestamp, prefix) in SIZE_CONVERSIONS:
		if width == 0:
			width = orig_width
		if height == 0:
			height = orig_height
		pathname_new = os.path.join(dirname, ''.join([prefix, timestamp_name, FILETYPE]))
		if timestamp:
			child = subprocess.Popen(
			                          [
			                            IM_CONVERT,
			                            pathname,
			                            '-resize',
			                            ''.join([str(width), 'x', str(height)]),
			                            '-font',
			                            TS_FONT,
			                            '-pointsize',
			                            str(TS_SIZE),
			                            '-fill',
			                            'black',
			                            '-stroke',
			                            'black',
			                            '-strokewidth',
			                            '2',
			                            '-annotate',
			                            ''.join(['+', str(TS_OFFSET_LEFT), '+', str(height - TS_OFFSET_BOTTOM - 1)]),
			                            display_timestamp,
			                            '-fill',
			                            'red',
			                            '-stroke',
			                            'none',
			                            '-annotate',
			                            ''.join(['+', str(TS_OFFSET_LEFT), '+', str(height - TS_OFFSET_BOTTOM - 1)]),
			                            display_timestamp,
			                            pathname_new
			                          ]
			                        )
			child.wait()
		else:
			child = subprocess.Popen(
			                          [
			                            IM_CONVERT,
			                            pathname,
			                            '-resize',
			                            ''.join([str(width), 'x', str(height)]),
			                            pathname_new
			                          ]
			                        )
			child.wait()

	# Search for scoreboard

	if SCOREBOARD_DEBUG:
		# Debug image (will replaced if a scoreboard is found
		pathname_debug = os.path.join(dirname, ''.join(['convert_debug_', timestamp_name, FILETYPE]))
		child = subprocess.Popen(
		                          [
		                            IM_CONVERT,
		                            pathname,
		                            pathname_debug
		                          ]
		                        )
		child.wait()

	# Skip small images
	if (
	     ( orig_width < 640 ) or
	     ( orig_height < 480 )
	   ):
		return

	scoreboard_info = {
	                    'min_x': orig_width,
	                    'min_y': orig_height,
	                    'max_x': 0,
	                    'max_y': 0,
	                    'header_count': 0,
	                    'client_count': 0,
	                    'is_ctf': False,
	                    'is_ctl': False
	                  }
	column_width = 160

	# Left side scoreboard
	scoreboard_topleft_x = ( orig_width / 2 ) - column_width
	scoreboard_topleft_y = 56 + ( ( orig_height - 480 ) / 2 )
	check_scoreboard(pathname, scoreboard_topleft_x, scoreboard_topleft_y, scoreboard_info)

	# Right side scoreboard
	scoreboard_topleft_x = orig_width / 2
	scoreboard_topleft_y = 56 + ( ( orig_height - 480 ) / 2 )
	found_old = scoreboard_info['header_count'] + scoreboard_info['client_count']
	check_scoreboard(pathname, scoreboard_topleft_x, scoreboard_topleft_y, scoreboard_info)

	# Right side scoreboard for team_red_blue modes
	extra_wide = False
	if ( ( scoreboard_info['header_count'] + scoreboard_info['client_count'] ) == found_old ):
		scoreboard_topleft_x = ( orig_width / 2 ) + 20
		scoreboard_topleft_y = 56 + ( ( orig_height - 480 ) / 2 )
		found_old = scoreboard_info['header_count'] + scoreboard_info['client_count']
		check_scoreboard(pathname, scoreboard_topleft_x, scoreboard_topleft_y, scoreboard_info)
		if ( ( scoreboard_info['header_count'] + scoreboard_info['client_count'] ) != found_old ):
			extra_wide = True

	if (
	     ( extra_wide ) or
	     ( scoreboard_info['is_ctf'] ) or
	     ( scoreboard_info['is_ctl'] )
	   ):
		column_width = column_width + 20

	if (
	     ( scoreboard_info['header_count'] != 0 ) and
	     ( scoreboard_info['client_count'] == 0 )
	   ):
		sys.stderr.write('Error: Header found without clients\n')

	if ( scoreboard_info['client_count'] > 0 ):
		# Calculate scoreboard dimensions
		width = scoreboard_info['max_x'] - scoreboard_info['min_x'] + column_width
		height = scoreboard_info['max_y'] - scoreboard_info['min_y'] + 32
		x = scoreboard_info['min_x']
		y = scoreboard_info['min_y']
		if ( scoreboard_info['is_ctf'] ):
			x = x - 32
			width = width + 32

		if SCOREBOARD_DEBUG:
			# Debug image
			child = subprocess.Popen(
			                          [
			                            IM_CONVERT,
			                            pathname,
			                            '-fill',
			                            'none',
			                            '-stroke',
			                            'yellow',
			                            '-draw',
			                            ''.join(['rectangle ', str(x - 1), ',', str(y - 1), ' ', str(x + width), ',', str(y + height)]),
			                            pathname_debug
			                          ]
			                        )
			child.wait()

		# Add borders
		width = width + BORDER_LEFT + BORDER_RIGHT
		height = height + BORDER_TOP + BORDER_BOTTOM
		x = x - BORDER_LEFT
		y = y - BORDER_TOP

		# Scoreboard image
		pathname_new = os.path.join(dirname, ''.join(['convert_scoreboard_', timestamp_name, FILETYPE]))
		child = subprocess.Popen(
		                          [
		                            IM_CONVERT,
		                            pathname,
		                            '-crop',
		                            ''.join([str(width), 'x', str(height), '+', str(x), '+', str(y)]),
		                            '+repage',
		                            pathname_new
		                          ]
		                        )
		child.wait()

		# Scoreboard image with timestamp

		# Place timestamp in bottom border; Enlarge border if it's too small
		ts_height = TS_OFFSET_BOTTOM + TS_SIZE + TS_OFFSET_BOTTOM
		if ( ts_height > BORDER_BOTTOM ):
			height = height + ts_height - BORDER_BOTTOM

		pathname_new = os.path.join(dirname, ''.join(['convert_ts_scoreboard_', timestamp_name, FILETYPE]))
		child = subprocess.Popen(
		                          [
		                            IM_CONVERT,
		                            pathname,
		                            '-crop',
		                            ''.join([str(width), 'x', str(height), '+', str(x), '+', str(y)]),
		                            '+repage',
		                            '-font',
		                            TS_FONT,
		                            '-pointsize',
		                            str(TS_SIZE),
		                            '-fill',
		                            'black',
		                            '-stroke',
		                            'black',
		                            '-strokewidth',
		                            '2',
		                            '-annotate',
		                            ''.join(['+', str(TS_OFFSET_LEFT), '+', str(height - TS_OFFSET_BOTTOM - 1)]),
		                            display_timestamp,
		                            '-fill',
		                            'red',
		                            '-stroke',
		                            'none',
		                            '-annotate',
		                            ''.join(['+', str(TS_OFFSET_LEFT), '+', str(height - TS_OFFSET_BOTTOM - 1)]),
		                            display_timestamp,
		                            pathname_new
		                          ]
		                        )
		child.wait()


def process_dir(dirname):
	dirname = os.path.normpath(dirname)

	if ( not os.path.isdir(dirname) ):
		sys.stderr.write('Skipping %s: Not a directory\n' % (dirname,))
		return

	png_screenshots = glob.glob(os.path.join(dirname, 'sof_[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]_[0-9][0-9][0-9][0-9][0-9][0-9]*.png'))
	for png_screenshot in png_screenshots:
		sys.stdout.write('Converting %s\n' % (png_screenshot,))
		convert(png_screenshot)


if __name__ == '__main__':
	if ( len(sys.argv) > 1 ):
		# Process specified directories
		for dirname in sys.argv[1:]:
			process_dir(dirname)
	else:
		# Process current directory
		process_dir('.')
